@@ -137,28 +137,27 @@ def enumerateSimple(collection, name='ENUM'):
     return ee.ImageCollection(fc.copyProperties(source=collection))
 
 
-def fillWithLast(collection):
+def fillWithLast(collection, unmask_first=False):
     """ Fill masked values of each image pixel with the last available
     value
 
     :param collection: the collection that holds the images that will be filled
     :type collection: ee.ImageCollection
+    :param unmask_first: unmask first image?
+    :type unmask_first: bool
     :rtype: ee.ImageCollection
     """
-
-    asc = collection.sort('system:time_start')
-    first_date = asc.first().date()
-
-    def wrap(img):
-        img = ee.Image(img)
-        d = img.date()
-        condition = d.difference(first_date, 'second')
-        def true(date):
-            newcol = collection.filterDate(first_date, date.advance(1, 'day'))
-            return composite.closestDate(newcol)
-        return ee.Image(ee.Algorithms.If(condition, true(d), img))
-
-    return collection.map(wrap)
+    collector = ee.List([])
+    def overcol(i, collect):
+        collect = ee.List(collect)
+        def true():
+            last = ee.Image(collect.get(-1))
+            mask = i.mask().Not()
+            return collect.add(ee.Image(i.unmask().where(mask, last)))
+        def false():
+            return collect.add(i.unmask() if unmask_first else i)
+        return ee.List(ee.Algorithms.If(collect.size(), true(), false()))
+    return ee.ImageCollection.fromImages(collection.iterate(overcol, collector))
 
 
 def mergeGeometries(collection):
@@ -188,22 +187,18 @@ def mosaicSameDay(collection, qualityBand=None):
         kept is `system:time_start`
     :rtype: ee.ImageCollection
     """
-    def make_date_list(img, l):
+    all_dates = collection.aggregate_array('system:time_start')
+    def overdates(d, l):
         l = ee.List(l)
-        img = ee.Image(img)
-        date = img.date()
-        # make clean date
+        date = ee.Date(d)
         day = date.get('day')
         month = date.get('month')
         year = date.get('year')
         clean_date = ee.Date.fromYMD(year, month, day)
         condition = l.contains(clean_date)
-
         return ee.Algorithms.If(condition, l, l.add(clean_date))
 
-    col_list = collection.toList(collection.size())
-    date_list = ee.List(col_list.iterate(make_date_list, ee.List([])))
-
+    date_list = ee.List(all_dates.iterate(overdates, ee.List([])))
     first_img = ee.Image(collection.first())
     bands = first_img.bandNames()
 
@@ -334,6 +329,43 @@ def makeEqualInterval(collection, interval=1, unit='month'):
     return imlist
 
 
+def makeDayIntervals(collection, interval=30, reverse=False, buffer='second'):
+    """ Make day intervals """
+    interval = int(interval)
+    collection = collection.sort('system:time_start', True)
+    start = collection.first().date()
+    end = collection.sort('system:time_start', False).first().date()
+    ranges = date.dayRangeIntervals(start, end, interval, reverse, buffer)
+
+    def over_ranges(drange, ini):
+        ini = ee.List(ini)
+        drange = ee.DateRange(drange)
+        start = drange.start()
+        end = drange.end()
+        filtered = collection.filterDate(start, end)
+        condition = ee.Number(filtered.size()).gt(0)
+        return ee.List(ee.Algorithms.If(condition, ini.add(filtered), ini))
+
+    imlist = ee.List(ranges.iterate(over_ranges, ee.List([])))
+
+    return imlist
+
+
+def reduceDayIntervals(collection, reducer, interval=30, reverse=False,
+                       buffer='second'):
+    """ Reduce Day Intervals
+
+    :param reducer: a function that takes as only argument a collection
+        and returns an image
+    :type reducer: function
+    :return: an image collection
+    :rtype: ee.ImageCollection
+    """
+    intervals = makeDayIntervals(collection, interval, reverse, buffer)
+    reduced = intervals.map(reducer)
+    return ee.ImageCollection.fromImages(reduced)
+
+
 def getValues(collection, geometry, scale=None, reducer=None,
               id='system:index', properties=None, side='server',
               maxPixels=1e9):
